/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

package com.fundi.framework.common.displaylist;

import java.util.LinkedHashMap;
import java.util.Map;

import com.fundi.framework.common.displaylist.PropDefn.EditResult;
import com.fundi.framework.common.displaylist.PropDefn.ValidationStrategy;
import com.fundi.framework.common.nl1.Messages;

public class ResourceDefn {
	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$

	private String					name = Messages.getString("ResourceDefn.0"); //$NON-NLS-1$
	private String					displayName = Messages.getString("ResourceDefn.1"); //$NON-NLS-1$
	private String					pluralName = Messages.getString("ResourceDefn.2"); //$NON-NLS-1$
	private Map<String, PropDefn>	props = new LinkedHashMap<String, PropDefn>();
	
	public enum ResourceType {
		DB("DB", ValidationStrategy.NAME_ALPHA_START), 		 //$NON-NLS-1$
        DBDESC("DBDESC", ValidationStrategy.NAME_ALPHA_START),	 //$NON-NLS-1$
	    PGM("PGM", ValidationStrategy.NAME_ALPHA_START), 	 //$NON-NLS-1$
	    PGMDESC("PGMDESC", ValidationStrategy.NAME_ALPHA_START),	 //$NON-NLS-1$
	    RTC("RTC", ValidationStrategy.NAME), 	 //$NON-NLS-1$
	    RTCDESC("RTCDESC", ValidationStrategy.NAME),	 //$NON-NLS-1$
	    TRAN("TRAN", ValidationStrategy.NAME), 	 //$NON-NLS-1$
	    TRANDESC("TRANDESC", ValidationStrategy.NAME); //$NON-NLS-1$
		//
		private final ValidationStrategy 	vs;
		private final String 				typeName;
		//
		private ResourceType(String typeName, ValidationStrategy vs) {
			this.typeName = typeName;
			this.vs = vs;
		}
		public EditResult validateName(String name) {
			return vs.validateName(name);
		}
		public String getTypeName() {
			return typeName;
		}
		@Override
		public String toString() {
			return typeName;
		}
	}
	
	public static ResourceType getResourceType(String typeName) {
		for (ResourceType type: ResourceType.values()) {
			if (type.typeName.equals(typeName)) {
				return type;
			}
		}
		
		return null;
	}
	
	/**
	 * Holds a list of resource types, each resource type MAY have a related set of property definitions, though this is not required to be populated.
	 */
	public ResourceDefn(String name, String displayName, String pluralName) {
		this.name = name;
		this.displayName = displayName;
		this.pluralName = pluralName;
	}

	public PropDefn getPropDefnFor(ListFieldDefn fieldDefn) {
		PropDefn	result;
		String		keyLabel;
		
		result = null;
		for (PropDefn pdefn: props.values()) {
			if (pdefn.getKeyLabel().equals(fieldDefn.getKeyLabel())) {
				result = pdefn;
				break;
			}
		}
		
		return result;
	}

	@Override
	public String toString() {
		return super.toString();
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDisplayName() {
		return displayName;
	}

	public void setDisplayName(String displayName) {
		this.displayName = displayName;
	}

	public Map<String, PropDefn> getProps() {
		return props;
	}

	public void setProps(Map<String, PropDefn> props) {
		this.props = props;
	}

	public PropDefn getPropDefn(String propsName) {
		return props.get(propsName);
	}
	
	public void setPropDefn(PropDefn pdefn) {
		props.put(pdefn.getPropsName(), pdefn);
	}

	public String getPluralName() {
		return pluralName;
	}

	public void setPluralName(String pluralName) {
		this.pluralName = pluralName;
	}
	
	public boolean isEditable() {
		for (PropDefn prop: props.values()) {
			if (prop.isEditable()) {
				return true;
			}
		}
		
		return false;
	}
}
