package com.fundi.framework.common.instance;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;

import com.fundi.framework.common.navigation.INavigationType;
import com.fundi.framework.common.navigation.NavigationItem;

public class NavigationItemStatusesThread extends Thread {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private AppInstance instance;
	private ArrayList<NavigationItem> allItems;
	private ArrayList<NavigationItem> groupedItems;
	
	public NavigationItemStatusesThread(AppInstance instance, ArrayList<NavigationItem> allItems, ArrayList<NavigationItem> groupedItems) {
		
		super();
		
		this.instance = instance;
		this.allItems = allItems;
		this.groupedItems = groupedItems;
    }

	public void run() {
	
		int idxItm;
		NavigationItem item;
		int bestStatus;
		INavigationType type;
		int idxAlt;
		ArrayList<Object> altData;
		
		/*
		 *  The primary data object may not be the one that can directly connect to anything, e.g. if GPL provides the ICON definition but it is CEX
		 *  which provides the driver that connects to ICONs.
		 *  
		 *  Test for connectability against the data object, then if it fails against any alternate data objects.
		 *  Stop if any one gets a good connection status.  Also a definite fail takes precedence over an unknown status.
		 */
		
		for (idxItm = 0; idxItm < allItems.size(); idxItm++) {
			item = allItems.get(idxItm);
			if (INavigationType.class.isInstance(item.getData())) {
				type = (INavigationType) item.getData();
				type.testConnection(instance);
				bestStatus = type.getConnectStatus();
				if (bestStatus != AppInstance.CAN_CONNECT_OK) {
					altData = item.getAlternateData();
					for (idxAlt = 0; idxAlt < altData.size(); idxAlt++) {
						if (INavigationType.class.isInstance(altData.get(idxAlt))) {
							type = (INavigationType) altData.get(idxAlt);
							type.testConnection(instance);
							if (type.getConnectStatus() == AppInstance.CAN_CONNECT_OK) {
								bestStatus = type.getConnectStatus();
								break;
							}
							else {
								if (type.getConnectStatus() > bestStatus) {
									bestStatus = type.getConnectStatus();
								}
							}
						}
					}
				}
				((INavigationType) item.getData()).setConnectStatus(bestStatus);
			}
		}
		
		instance.notifyNavigationListeners(groupedItems);
		
		instance.setFirstTimeGettingStatuses(false);
	}
}
